// includes, system
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

// includes, project
#include <cutil.h>

// includes, kernels
#include <BugReport01_kernel.cu>

////////////////////////////////////////////////////////////////////////////////
// declaration
void runTest( int argc, char** argv);


////////////////////////////////////////////////////////////////////////////////
// Program main
////////////////////////////////////////////////////////////////////////////////
int
main( int argc, char** argv) 
{
    runTest( argc, argv);

    CUT_EXIT(argc, argv);
}

////////////////////////////////////////////////////////////////////////////////
// Run a test
////////////////////////////////////////////////////////////////////////////////
void
runTest( int argc, char** argv) 
{
    // Number of threads
    unsigned int num_threads = 48;

    // operand length
    unsigned int operand_length = 1024;

    // allocated memory
    unsigned int mem_size = sizeof(unsigned int) * num_threads * operand_length;

    // shared memory size
    unsigned int shared_length = sizeof(unsigned int) * num_threads;


    CUT_DEVICE_INIT();

    // Allocate host memory
    unsigned int* host_input = (unsigned int*) malloc( mem_size);

    // initialize the memory
    for (unsigned int i = 0; i < (num_threads * operand_length); ++i) 
    {
        host_input[i] = 0;
    }
    // some values are != 0
    host_input[10] = 1;
    host_input[3] = 4;
    
    
    // Allocate device memory
    unsigned int* device_input;

    CUDA_SAFE_CALL( cudaMalloc( (void**) &device_input, mem_size));

    // Copy host memory to device
    CUDA_SAFE_CALL( cudaMemcpy( device_input, host_input, mem_size, cudaMemcpyHostToDevice) );


    // Setup execution parameters
    dim3  grid( 1, 1, 1);
    dim3  threads( num_threads, 1, 1);

    // Der Kernel wird ausgeführt
    MyKernel<<< grid, threads, shared_length >>>( device_input, operand_length);

    // Check if the kernel execution generated an error
    CUT_CHECK_ERROR("*****Kernel execution failed*****");

    // Free memory
    free(host_input);
    CUDA_SAFE_CALL(cudaFree(device_input));
}
