#ifndef _TEMPLATE_KERNEL_H_
#define _TEMPLATE_KERNEL_H_

#include <stdio.h>


__global__ void
MyKernel(unsigned int* g_input, unsigned int operand_length)
{

    // Shared Memory (quite small -  num_threads * sizeof( int) )
    extern  __shared__  int shared_array[];

    // Thread ID
    const unsigned int tid = threadIdx.x;
    
    // Number of threads
    const unsigned int num_threads = blockDim.x;

    // each thread gets its own portion of data
    unsigned int pointer_position;

    // if all members of array 'g_input' == 0 then is_not_null will be 0
    // else is_not_null will be 1
    unsigned int is_not_null;
    
    // each thread writes results to dedicated position of shared_array (shared_array[tid])
    shared_array[tid] = 0;

    for (unsigned int i = 0; i < operand_length; ++i)
    {
	// each thread gets its own portion of data
	pointer_position = tid * operand_length + i;

	// if at least one  member of array is !=0 then
	if (g_input[pointer_position] != 0)
	{
	    // write an 1 into shared_array[tid] ...
	    shared_array[tid] = 1;
	    // ... and exit the loop

	    // This is one of two 'break' in my code that cause malfunctions
	    // if you remove it the kernel will work perfectly
	    break;
	}
     }
    __syncthreads();


    // The first thread (with tid = 0) collects all results from 'shared_array'
    // Other threads have nothing to do
    if (tid == 0)
    {
	is_not_null = 0;
	for (unsigned int i = 0; i < num_threads; ++i)
	{
	    // If at least one thread detected an number != 0 in array 'g_input'
	    // (at least one member of shared_array is 1)
	    if (shared_array[i] != 0)
	    {
		// at least one member of array is not null !
		is_not_null = 1;
		// exit the loop

		// This is one of two 'break' in my code that cause malfunctions
		// if you remove it the kernel will work perfectly
		break;
	    }
	}
    }
    __syncthreads();
}

#endif // #ifndef _TEMPLATE_KERNEL_H_
